package com.gmei.function;

import com.gmei.bean.bl.BlPreciseExposureBean;
import com.gmei.cache.PreciseAccumulator;
import com.gmei.utils.MurmurHash;
import org.apache.flink.api.common.state.StateTtlConfig;
import org.apache.flink.api.common.state.ValueState;
import org.apache.flink.api.common.state.ValueStateDescriptor;
import org.apache.flink.api.common.time.Time;
import org.apache.flink.api.common.typeinfo.TypeInformation;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.KeyedProcessFunction;
import org.apache.flink.util.Collector;
import org.roaringbitmap.longlong.Roaring64NavigableMap;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class BitMapFunction extends KeyedProcessFunction<String, BlPreciseExposureBean,BlPreciseExposureBean> {
    private static final long serialVersionUID = 1L;
    private static final Logger LOGGER = LoggerFactory.getLogger(BitMapFunction.class);
//    private volatile PreciseAccumulator bitMap;
    private PreciseAccumulator defaultValue;
    private ValueState<PreciseAccumulator> bitMapState;
    private ValueStateDescriptor<PreciseAccumulator> bitMapStateDesc;

    @Override
    public void onTimer(long timestamp, OnTimerContext ctx, Collector<BlPreciseExposureBean> out) throws Exception {
        super.onTimer(timestamp, ctx, out);
        bitMapState.clear();
//        bitMap = new PreciseAccumulator();
    }

    @Override
    public void open(Configuration parameters) throws Exception {
        super.open(parameters);
//        bitMap = new PreciseAccumulator();
        //设置ValueState的TTL的生命周期为24个小时，自动会清除ValueState的里内容
        StateTtlConfig ttlConfig = StateTtlConfig
                .newBuilder(Time.days(1))
                .setUpdateType(StateTtlConfig.UpdateType.OnCreateAndWrite)
                .setStateVisibility(StateTtlConfig.StateVisibility.NeverReturnExpired)
                .build();
        bitMapStateDesc = new ValueStateDescriptor<PreciseAccumulator>("bitMap", TypeInformation.of(PreciseAccumulator.class));
        bitMapStateDesc.enableTimeToLive(ttlConfig);
        bitMapState = getRuntimeContext().getState(bitMapStateDesc);
    }

    @Override
    public void close() throws Exception {
        super.close();
//        bitMap.clear();
        bitMapState.clear();
    }

    @Override
    public void processElement(BlPreciseExposureBean blPreciseExposureBean, Context context, Collector<BlPreciseExposureBean> collector) throws Exception {
        String blPreciseExposureBeanId = blPreciseExposureBean.getJson();
        long hashCode = MurmurHash.hash64(blPreciseExposureBeanId);
        if (this.bitMapState.value() == null) {
            defaultValue = new PreciseAccumulator(0);
            defaultValue.add(0);
            bitMapState.update(defaultValue);
        }
        PreciseAccumulator bitMapValue = this.bitMapState.value();
        if (!bitMapValue.contains(hashCode)) {
            bitMapValue.add(hashCode);
            bitMapState.update(bitMapValue);
//            bitMap.add(hashCode);
            collector.collect(blPreciseExposureBean);
        } else {
            System.out.println(blPreciseExposureBean);
        }
//        context.timerService().registerProcessingTimeTimer(DateUtil.tomorrowZeroTimestampMs(Double.valueOf(blPreciseExposureBean.getGm_nginx_timestamp()).longValue() * 1000, 8) + 1);
    }
}
